@@ -4,211 +4,106 @@
 //! provide a way to describe how resources should be shared and used to produce a single
 //! pixel frame.
 //!
-//! # Pipelines and AST
+//! # Pipelines and graphs
 //!
 //! luminance has a very particular way of doing graphics. It represents a typical _graphics
-//! pipeline_ via a typed [AST] that is embedded into your code. As you might already know, when you
-//! write code, you’re actually creating an [AST]: expressions, assignments, bindings, conditions,
-//! function calls, etc. They all represent a typed tree that represents your program.
+//! pipeline_ via a typed graph that is embedded into your code. Graphs are used to create a
+//! dependency between resources your GPU needs to have in order to perform a render. It might
+//! be weird at first but you’ll see how simple and easy it actually is. If you want to perform
+//! a simple draw call of a triangle, you need several resources:
 //!
-//! luminance uses that property to create a dependency between resources your GPU needs to
-//! have in order to perform a render. It might be weird at first but you’ll see how simple and easy
-//! it is. If you want to perform a simple draw call of a triangle, you need several resources:
-//!
-//! - A [`Tess`] that represents the triangle. It holds three vertices.
+//! - A [`Tess`] that represents the triangle — assuming you don’t cheat with an attributeless
+//!   render ;). It holds three vertices.
 //! - A shader [`Program`], for shading the triangle with a constant color, for short and simple.
 //! - A [`Framebuffer`], to accept and hold the actual render.
 //! - A [`RenderState`], to state how the render should be performed.
 //! - And finally, a [`PipelineState`], which allows even more customization on how the pipeline
-//!   behaves
+//!   runs.
 //!
-//! There is a dependency _graph_ to represent how the resources must behave regarding each other:
+//! The terminology used in luminance is as follows: a graphics pipeline is a graph in which nodes
+//! are called _gates_. A gate represents a particular resource exposed as a shared scarce resource
+//! for child, nested nodes. For instance, you can share framebuffer in a [`PipelineGate`]: all
+//! nodes that are children of that framebuffer node will then be able to render into that
+//! framebuffer.
 //!
-//! ```text
-//! (AST1)
-//!
-//! PipelineState ─── Framebuffer ─── Shader ─── RenderState ─── Tess
-//! ```
-//!
-//! The framebuffer must be _active_, _bound_, _used_ — or whatever verb you want to picture it
-//! with — before the shader can start doing things. The shader must also be in use before we can
-//! actually render the tessellation.
-//!
-//! That triple dependency relationship is already a small flat [AST]. Imagine we want to render
-//! a second triangle with the same render state and a third triangle with a different render state:
-//!
-//! ```text
-//! (AST2)
-//!
-//! PipelineState ─── Framebuffer ─── Shader ─┬─ RenderState ─┬─ Tess
-//!                                           │               │
-//!                                           │               └─ Tess
-//!                                           │
-//!                                           └─ RenderState ─── Tess
-//! ```
-//!
-//! That [AST] looks more complex. Imagine now that we want to shade one other triangle with
-//! another shader!
-//!
-//! ```text
-//! (AST3)
-//!
-//! PipelineState ─── Framebuffer ─┬─ Shader ─┬─ RenderState ─┬─ Tess
-//!                                │          │               │
-//!                                │          │               └─ Tess
-//!                                │          │
-//!                                │          └─ RenderState ─── Tess
-//!                                │
-//!                                └─ Shader ─── RenderState ─── Tess
-//! ```
-//!
-//! You can now clearly see the [AST]s and the relationships between objects. Those are encoded
-//! in luminance within your code directly: lambdas / closures.
-//!
-//! > If you have followed thoroughly, you might have noticed that you cannot, with such [AST]s,
-//! > shade a triangle with another shader but using the same render state as another node. That
-//! > was a decision that was needed to be made: how should we allow the [AST] to be shared?
-//! > In terms of graphics pipeline, luminance tries to do the best thing to minimize the number
-//! > of GPU context switches and CPU <=> GPU bandwidth congestion.
-//!
-//! # The lambda & closure design
-//!
-//! A function is a perfect candidate to modelize a dependency: the arguments of the function
-//! modelize the dependency — they will be provided _at some point in time_, but it doesn’t matter
-//! when while writing the function. We can then write code _depending_ on something without even
-//! knowing where it’s from.
-//!
-//! Using pseudo-code, here’s what the ASTs from above look like: (this is not a real luminance,
-//! excerpt, just a simplification).
-//!
-//! ```ignore
-//! // AST1
-//! pipeline(framebuffer, pipeline_state, || {
-//!   // here, we are passing a closure that will get called whenever the framebuffer is ready to
-//!   // receive renders
-//!   use_shader(shader, || {
-//!     // same thing but for shader
-//!     use_render_state(render_state, || {
-//!       // ditto for render state
-//!       triangle.render(); // render the tessellation
-//!     });
-//!   );
-//! );
-//! ```
-//!
-//! See how simple it is to represent `AST1` with just closures? Rust’s lifetimes and existential
-//! quantification allow us to ensure that no resource will leak from the scope of each closures,
-//! hence enforcing memory and coherency safety.
-//!
-//! Now let’s try to tackle `AST2`.
-//!
-//! ```ignore
-//! // AST2
-//! pipeline(framebuffer, pipeline_state, || {
-//!   use_shader(shader, || {
-//!     use_render_state(render_state, || {
-//!       first_triangle.render();
-//!       second_triangle.render(); // simple and straight-forward
-//!     });
-//!
-//!     // we can just branch a new render state here!
-//!     use_render_state(other_render_state, || {
-//!       third.render()
-//!     });
-//!   );
-//! );
-//! ```
-//!
-//! And `AST3`:
-//!
-//! ```ignore
-//! // AST3
-//! pipeline(framebuffer, pipeline_state, || {
-//!   use_shader(shader, || {
-//!     use_render_state(render_state, || {
-//!       first_triangle.render();
-//!       second_triangle.render(); // simple and straight-forward
-//!     });
-//!
-//!     // we can just branch a new render state here!
-//!     use_render_state(other_render_state, || {
-//!       third.render()
-//!     });
-//!   );
-//!
-//!   use_shader(other_shader, || {
-//!     use_render_state(yet_another_render_state, || {
-//!       other_triangle.render();
-//!     });
-//!   });
-//! );
-//! ```
-//!
-//! The luminance equivalent is a bit more complex because it implies some objects that need
-//! to be introduced first.
+//! This design is well-typed: when you enter a given gate, the set of operations and children nodes
+//! you can create is directly dependent on the parent node.
 //!
 //! # PipelineGate and Pipeline
 //!
-//! A [`PipelineGate`] represents a whole [AST] as seen as just above. It is created by a
-//! [`GraphicsContext`] when you ask to create a pipeline gate. A [`PipelineGate`] is typically
+//! A [`PipelineGate`] is the gate allowing to share a [`Framebuffer`].
+//!
+//! A [`PipelineGate`] represents a whole graphics pipeline as seen as just above. It is created by
+//! a [`GraphicsContext`] when you ask to create a pipeline gate. A [`PipelineGate`] is typically
 //! destroyed at the end of the current frame, but that’s not a general rule.
 //!
-//! Such an object gives you access, via the [`PipelineGate::pipeline`], to two other objects
-//! :
+//! Such an object gives you access, via the [`PipelineGate::pipeline`], to two other objects:
 //!
 //! - A [`ShadingGate`], explained below.
 //! - A [`Pipeline`].
 //!
-//! A [`Pipeline`] is a special object you can use to use some specific scarce resources, such as
-//! _textures_ and _buffers_. Those are treated a bit specifically on the GPU, so you have to use
-//! the [`Pipeline`] interface to deal with them.
+//! A [`Pipeline`] is a special object you can use to handle some specific scarce resources, such as
+//! _textures_ and _shader data. Those are treated a bit specifically on the backend, so you have to
+//! use the [`Pipeline`] interface to deal with them. Those scarce resources can be shared at different
+//! depth in the graphics pipeline, which is the reason why they are exposed via this [`Pipeline`]
+//! object, that you can pass down the graph.
 //!
 //! Creating a [`PipelineGate`] requires two resources: a [`Framebuffer`] to render to, and a
-//! [`PipelineState`], allowing to customize how the pipeline will perform renders at runtime.
+//! [`PipelineState`], allowing to customize how the pipeline will perform renders at runtime. This gate
+//! will then do a couple of things on the backend, depending mainly on the [`PipelineState`] you pass.
+//! For instance, framebuffer clearing, sRGB conversion or scissor test is done at that level.
 //!
 //! # ShadingGate
 //!
-//! When you create a pipeline, you’re also handed a [`ShadingGate`]. A [`ShadingGate`] is an object
-//! that allows you to create _shader_ nodes in the [AST] you’re building. You have no other way
-//! to go deeper in the [AST].
+//! A [`ShadingGate`] is the gate allowing to share a shader [`Program`].
 //!
-//! That node will typically borrow a shader [`Program`] and will move you one level lower in the
-//! graph ([AST]). A shader [`Program`] is typically an object you create at initialization or at
-//! specific moment in time (i.e. you don’t create them each frame) that tells the GPU how vertices
+//! When you enter a [`PipelineGate`], you’re handed a [`ShadingGate`]. A [`ShadingGate`] is an object
+//! that allows you to create _shader_ nodes in the graphics pipeline. You have no other way to go deeper
+//! in the graph.
+//!
+//! A shader [`Program`] is typically an object you create at initialization or at specific moment in time
+//! (i.e. you don’t create them on each frame, that would be super costly) that tells the GPU how vertices
 //! should be transformed; how primitives should be moved and generated, how tessellation occurs and
 //! how fragment (i.e. pixels) are computed / shaded — hence the name.
 //!
-//! At that level (i.e. in that closure), you are given two objects:
+//! At that level (i.e. in that closure), you are given three objects:
 //!
-//!   - A [`RenderGate`], discussed below.
-//!   - A [`ProgramInterface`], which has as type parameter the type of uniform your shader
-//!     [`Program`] defines.
+//! - A [`RenderGate`], discussed below.
+//! - A [`ProgramInterface`], which has as type parameter the type of uniform your shader
+//!   [`Program`] defines.
+//! - The uniform interface the [`Program`] was made to work with.
 //!
 //! The [`ProgramInterface`] is the only way for you to access your _uniform interface_. More on
 //! this in the dedicated section. It also provides you with the [`ProgramInterface::query`]
 //! method, that allows you to perform _dynamic uniform lookup_.
 //!
+//! Once you have entered this gate, you know that everything nested will be shaded with the shared shader
+//! [`Program`].
+//!
 //! # RenderGate
 //!
+//! A [`RenderGate`] is the gate allowing to prepare renders by sharing [`RenderState`].
+//!
 //! A [`RenderGate`] is the second to last gate you will be handling. It allows you to create
-//! _render state_ nodes in your [AST], creating a new level for you to render tessellations with
+//! _render state_ nodes in your graph, creating a new level for you to render tessellations with
 //! an obvious, final gate: the [`TessGate`].
 //!
 //! The kind of object that node manipulates is [`RenderState`]. A [`RenderState`] — a bit like for
 //! [`PipelineGate`] with [`PipelineState`] — enables to customize how a render of a specific set
-//! of objects (i.e. tessellations) will occur. It’s a bit more specific to renders than pipelines.
+//! of objects (i.e. tessellations) will occur. It’s a bit more specific to renders than pipelines and
+//! will allow customizing aspects like _blending_, _depth test_, _backface culling_, etc.
 //!
 //! # TessGate
 //!
-//! The [`TessGate`] is the final gate you use in an [AST]. It’s used to create _tessellation
-//! nodes_. Those are used to render actual [`Tess`]. You cannot go any deeper in the [AST] at that
-//! stage.
+//! A [`TessGate`] is the final gate, allowing to share [`Tess`].
+//!
+//! The [`TessGate`] is the final gate you use in a graphics pipeline. It’s used to create _tessellation
+//! nodes_. Those are used to render actual [`Tess`]. You cannot go any deeper in the graph at that stage.
 //!
 //! [`TessGate`]s don’t immediately use [`Tess`] as inputs. They use [`TessView`]. That type is
-//! a simple GPU view into a GPU tessellation ([`Tess`]). It can be obtained from a [`Tess`] via
-//! the [`View`] trait or built explicitly.
+//! a simple immutable view into a [`Tess`]. It can be obtained from a [`Tess`] via the [`View`] trait or
+//! built explicitly.
 //!
-//! [AST]: https://en.wikipedia.org/wiki/Abstract_syntax_tree
 //! [`Tess`]: crate::tess::Tess
 //! [`Program`]: crate::shader::Program
 //! [`Framebuffer`]: crate::framebuffer::Framebuffer
@@ -231,15 +126,15 @@ use std::{
 use crate::{
   backend::{
     color_slot::ColorSlot,
-    depth_slot::DepthSlot,
+    depth_stencil_slot::DepthStencilSlot,
     framebuffer::Framebuffer as FramebufferBackend,
-    pipeline::{Pipeline as PipelineBackend, PipelineBase, PipelineBuffer, PipelineTexture},
+    pipeline::{Pipeline as PipelineBackend, PipelineBase, PipelineShaderData, PipelineTexture},
   },
-  buffer::Buffer,
   context::GraphicsContext,
   framebuffer::Framebuffer,
   pixel::Pixel,
   scissor::ScissorRegion,
+  shader::ShaderData,
   shading_gate::ShadingGate,
   texture::{Dimensionable, Texture},
 };
@@ -280,16 +175,39 @@ pub enum Viewport {
 #[non_exhaustive]
 #[derive(Clone, Debug)]
 pub struct PipelineState {
-  /// Color to use when clearing buffers.
-  pub clear_color: [f32; 4],
-  /// Whether clearing color buffers.
-  pub clear_color_enabled: bool,
-  /// Whether clearing depth buffers.
-  pub clear_depth_enabled: bool,
+  /// Color to use when clearing color buffers.
+  ///
+  /// Set this to `Some(color)` to use that color to clear the [`Framebuffer`] when running a [`PipelineGate`]. Set it
+  /// to `None` not to clear the framebuffer when running the [`PipelineGate`].
+  ///
+  /// An example of not setting the clear color is if you want to accumulate renders in a [`Framebuffer`] (for instance
+  /// for a paint-like application).
+  pub clear_color: Option<[f32; 4]>,
+
+  /// Depth value to use when clearing the depth buffer.
+  ///
+  /// Set this to `Some(depth)` to use that depth to clear the [`Framebuffer`] depth buffer.
+  pub clear_depth: Option<f32>,
+
+  /// Stencil value to use when clearing the stencil buffer.
+  ///
+  /// Set this to `Some(stencil)` to use that stencil to clear the [`Framebuffer`] stencil buffer.
+  pub clear_stencil: Option<i32>,
+
   /// Viewport to use when rendering.
   pub viewport: Viewport,
-  /// Whether [sRGB](https://en.wikipedia.org/wiki/SRGB) should be enabled.
+
+  /// Whether [sRGB](https://en.wikipedia.org/wiki/SRGB) support should be enabled.
+  ///
+  /// When this is set to `true`, shader outputs that go in [`Framebuffer`] for each of the color slots have sRGB pixel
+  /// formats are assumed to be in the linear RGB color space. The pipeline will then convert that linear color outputs
+  /// to sRGB to be stored in the [`Framebuffer`].
+  ///
+  /// Typical examples are when you are rendering into an image that is to be displayed to on screen: the
+  /// [`Framebuffer`] can use sRGB color pixel formats and the shader doesn’t have to worry about converting from linear
+  /// color space into sRGB color space, as the pipeline will do that for you.
   pub srgb_enabled: bool,
+
   /// Whether to use scissor test when clearing buffers.
   pub clear_scissor: Option<ScissorRegion>,
 }
@@ -297,17 +215,17 @@ pub struct PipelineState {
 impl Default for PipelineState {
   /// Default [`PipelineState`]:
   ///
-  /// - Clear color is `[0, 0, 0, 1]`.
-  /// - Color is always cleared.
-  /// - Depth is always cleared.
+  /// - Clear color is `Some([0., 0., 0., 1.])`.
+  /// - Depth value is `Some(1.)`.
+  /// - Stencil value is `Some(0)`.
   /// - The viewport uses the whole framebuffer’s.
   /// - sRGB encoding is disabled.
   /// - No scissor test is performed.
   fn default() -> Self {
     PipelineState {
-      clear_color: [0., 0., 0., 1.],
-      clear_color_enabled: true,
-      clear_depth_enabled: true,
+      clear_color: Some([0., 0., 0., 1.]),
+      clear_depth: Some(1.),
+      clear_stencil: Some(0),
       viewport: Viewport::Whole,
       srgb_enabled: false,
       clear_scissor: None,
@@ -323,41 +241,41 @@ impl PipelineState {
     Self::default()
   }
 
-  /// Get the clear color.
-  pub fn clear_color(&self) -> [f32; 4] {
-    self.clear_color
+  /// Get the clear color, if any.
+  pub fn clear_color(&self) -> Option<&[f32; 4]> {
+    self.clear_color.as_ref()
   }
 
   /// Set the clear color.
-  pub fn set_clear_color(self, clear_color: [f32; 4]) -> Self {
+  pub fn set_clear_color(self, clear_color: impl Into<Option<[f32; 4]>>) -> Self {
     Self {
-      clear_color,
+      clear_color: clear_color.into(),
       ..self
     }
   }
 
-  /// Check whether the pipeline’s framebuffer’s color buffers will be cleared.
-  pub fn is_clear_color_enabled(&self) -> bool {
-    self.clear_color_enabled
+  /// Get the clear depth, if any.
+  pub fn clear_depth(&self) -> Option<f32> {
+    self.clear_depth
   }
 
-  /// Enable clearing color buffers.
-  pub fn enable_clear_color(self, clear_color_enabled: bool) -> Self {
+  /// Set the clear depth.
+  pub fn set_clear_depth(self, clear_depth: impl Into<Option<f32>>) -> Self {
     Self {
-      clear_color_enabled,
+      clear_depth: clear_depth.into(),
       ..self
     }
   }
 
-  /// Check whether the pipeline’s framebuffer’s depth buffer will be cleared.
-  pub fn is_clear_depth_enabled(&self) -> bool {
-    self.clear_depth_enabled
+  /// Get the clear stencil, if any.
+  pub fn clear_stencil(&self) -> Option<i32> {
+    self.clear_stencil
   }
 
-  /// Enable clearing depth buffers.
-  pub fn enable_clear_depth(self, clear_depth_enabled: bool) -> Self {
+  /// Set the clear stencil.
+  pub fn set_clear_stencil(self, clear_stencil: impl Into<Option<i32>>) -> Self {
     Self {
-      clear_depth_enabled,
+      clear_stencil: clear_stencil.into(),
       ..self
     }
   }
@@ -421,30 +339,9 @@ impl<'a, B> Pipeline<'a, B>
 where
   B: PipelineBase,
 {
-  /// Bind a buffer.
-  ///
-  /// Once the buffer is bound, the [`BoundBuffer`] object has to be dropped / die in order to
-  /// bind the buffer again.
-  pub fn bind_buffer<T>(
-    &'a self,
-    buffer: &'a mut Buffer<B, T>,
-  ) -> Result<BoundBuffer<'a, B, T>, PipelineError>
-  where
-    B: PipelineBuffer<T>,
-    T: Copy,
-  {
-    unsafe {
-      B::bind_buffer(&self.repr, &buffer.repr).map(|repr| BoundBuffer {
-        repr,
-        _phantom: PhantomData,
-      })
-    }
-  }
-
   /// Bind a texture.
   ///
-  /// Once the texture is bound, the [`BoundTexture`] object has to be dropped / die in order to
-  /// bind the texture again.
+  /// Once the texture is bound, the [`BoundTexture`] object has to be dropped / die in order to bind the texture again.
   pub fn bind_texture<D, P>(
     &'a self,
     texture: &'a mut Texture<B, D, P>,
@@ -461,6 +358,25 @@ where
       })
     }
   }
+
+  /// Bind a shader data.
+  ///
+  /// Once the shader data is bound, the [`BoundShaderData`] object has to be dropped / die in order to bind the shader
+  /// data again.
+  pub fn bind_shader_data<T>(
+    &'a self,
+    shader_data: &'a mut ShaderData<B, T>,
+  ) -> Result<BoundShaderData<'a, B, T>, PipelineError>
+  where
+    B: PipelineShaderData<T>,
+  {
+    unsafe {
+      B::bind_shader_data(&self.repr, &shader_data.repr).map(|repr| BoundShaderData {
+        repr,
+        _phantom: PhantomData,
+      })
+    }
+  }
 }
 
 /// Top-most node in a graphics pipeline.
@@ -472,17 +388,11 @@ where
 /// # Parametricity
 ///
 /// - `B`, the backend type.
-pub struct PipelineGate<'a, B>
-where
-  B: ?Sized,
-{
+pub struct PipelineGate<'a, B> {
   backend: &'a mut B,
 }
 
-impl<'a, B> PipelineGate<'a, B>
-where
-  B: ?Sized,
-{
+impl<'a, B> PipelineGate<'a, B> {
   /// Create a new [`PipelineGate`].
   pub fn new<C>(ctx: &'a mut C) -> Self
   where
@@ -516,7 +426,7 @@ where
     B: FramebufferBackend<D> + PipelineBackend<D>,
     D: Dimensionable,
     CS: ColorSlot<B, D>,
-    DS: DepthSlot<B, D>,
+    DS: DepthStencilSlot<B, D>,
     F: for<'b> FnOnce(Pipeline<'b, B>, ShadingGate<'b, B>) -> Result<(), E>,
     E: From<PipelineError>,
   {
@@ -593,28 +503,27 @@ impl<E> DerefMut for Render<E> {
   }
 }
 
-/// Opaque buffer binding.
+/// Opaque shader data binding.
 ///
-/// This type represents a bound [`Buffer`] via [`BoundBuffer`]. It can be used along with a
-/// [`Uniform`] to customize a shader’s behavior.
+/// This type represents a bound [`ShaderData`] via [`BoundShaderData`]. It can be used along with a [`Uniform`] to
+/// customize a shader’s behavior.
 ///
 /// # Parametricity
 ///
-/// - `T` is the type of the carried item by the [`Buffer`].
+/// - `T` is the type of the carried item by the [`ShaderData`].
 ///
 /// # Notes
 ///
-/// You shouldn’t try to do store / cache or do anything special with that value. Consider it
-/// an opaque object.
+/// You shouldn’t try to do store / cache or do anything special with that value. Consider it an opaque object.
 ///
 /// [`Uniform`]: crate::shader::Uniform
 #[derive(Debug)]
-pub struct BufferBinding<T> {
+pub struct ShaderDataBinding<T> {
   binding: u32,
   _phantom: PhantomData<*const T>,
 }
 
-impl<T> BufferBinding<T> {
+impl<T> ShaderDataBinding<T> {
   /// Access the underlying binding value.
   ///
   /// # Notes
@@ -625,47 +534,43 @@ impl<T> BufferBinding<T> {
   }
 }
 
-/// A _bound_ [`Buffer`].
+/// A _bound_ [`ShaderData`].
 ///
 /// # Parametricity
 ///
-/// - `B` is the backend type. It must implement [`PipelineBuffer`].
-/// - `T` is the type of the carried item by the [`Buffer`].
+/// - `B` is the backend type. It must implement [`ShaderData`](crate::backend::shader::ShaderData).
+/// - `T` is the carried item type.
 ///
 /// # Notes
 ///
-/// Once a [`Buffer`] is bound, it can be used and passed around to shaders. In order to do so,
-/// you will need to pass a [`BufferBinding`] to your [`ProgramInterface`]. That value is unique
-/// to each [`BoundBuffer`] and should always be asked — you shouldn’t cache them, for instance.
+/// Once a [`ShaderData`] is bound, it can be used and passed around to shaders. In order to do so, you will need to
+/// pass a [`ShaderDataBinding`] to your [`ProgramInterface`]. That value is unique to each [`BoundShaderData`] and
+/// should always be asked — you shouldn’t cache them, for instance.
 ///
-/// Getting a [`BufferBinding`] is a cheap operation and is performed via the
-/// [`BoundBuffer::binding`] method.
+/// Getting a [`ShaderDataBinding`] is a cheap operation and is performed via the [`BoundShaderData::binding`] method.
 ///
 /// [`ProgramInterface`]: crate::shader::ProgramInterface
-pub struct BoundBuffer<'a, B, T>
+pub struct BoundShaderData<'a, B, T>
 where
-  B: PipelineBuffer<T>,
-  T: Copy,
+  B: PipelineShaderData<T>,
 {
-  pub(crate) repr: B::BoundBufferRepr,
-  _phantom: PhantomData<&'a T>,
+  pub(crate) repr: B::BoundShaderDataRepr,
+  _phantom: PhantomData<&'a ()>,
 }
 
-impl<'a, B, T> BoundBuffer<'a, B, T>
+impl<'a, B, T> BoundShaderData<'a, B, T>
 where
-  B: PipelineBuffer<T>,
-  T: Copy,
+  B: PipelineShaderData<T>,
 {
-  /// Obtain a [`BufferBinding`] object that can be used to refer to this bound buffer in shader
-  /// stages.
+  /// Obtain a [`ShaderDataBinding`] object that can be used to refer to this bound shader data in shader stages.
   ///
   /// # Notes
   ///
   /// You shouldn’t try to do store / cache or do anything special with that value. Consider it
   /// an opaque object.
-  pub fn binding(&self) -> BufferBinding<T> {
-    let binding = unsafe { B::buffer_binding(&self.repr) };
-    BufferBinding {
+  pub fn binding(&self) -> ShaderDataBinding<T> {
+    let binding = unsafe { B::shader_data_binding(&self.repr) };
+    ShaderDataBinding {
       binding,
       _phantom: PhantomData,
     }
