@@ -106,27 +106,14 @@
 //! very similar to environment variables. You can declare several ones as you would declare
 //! several environment variables. More on that on the documentation of [`Uniform`].
 //!
-//! ## Uniform buffers
+//! ## Shader data
 //!
-//! > This section is under heavy rewriting, both the documentation and API.
+//! Another way to pass data to shader is to use a [`ShaderData`]. This kind of object allows to
+//! shared data between shaders: set the data once, it will be available to any shader you pass the
+//! [`ShaderData`] to.
 //!
-//! Sometimes, you will want to set and pass around rich and more complex data. Instead of a `f32`,
-//! you will want to pass a `struct`. This operation is currently supported but highly unsafe. The
-//! reason for this is that your GPU will expect a specific kind of memory layout for the types
-//! you use, and that also depends on the backend you use.
-//!
-//! Also, passing a lot of data is not very practical with default [`Uniform`] directly.
-//!
-//! In order to pass more data or `struct`s, you need to create a [`Buffer`]. That buffer will
-//! simply contain the data / object(s) you want to pass to the shader. It is then possible, via
-//! the use of a [`Pipeline`], to retrieve a [`BoundBuffer`], which can be used to get a
-//! [`BufferBinding`]. That [`BufferBinding`] can then be set on a
-//! `Uniform<BufferBinding<YourType>>`, telling your shader program where to grab the data — from
-//! the bound buffer.
-//!
-//! This way of doing is very practical and powerful but currently, in this version of the crate,
-//! very unsafe. A better API will be available in a next release to make all this simpler and
-//! safer.
+//! Most implementation also allows much more data via this mechanism, allowing to pass huge amount
+//! of data to implement various techniques, such as _geometry instancing_ for instance.
 //!
 //! ## Uniform interfaces
 //!
@@ -136,18 +123,17 @@
 //! fields in it. More on the [`UniformInterface`] documentation.
 //!
 //! [`Vertex`]: crate::vertex::Vertex
-//! [`Buffer`]: crate::buffer::Buffer
 //! [`Pipeline`]: crate::pipeline::Pipeline
-//! [`BoundBuffer`]: crate::pipeline::BoundBuffer
-//! [`BufferBinding`]: crate::pipeline::BufferBinding
+//! [`ShaderData`]: crate::shader::ShaderData
 
-use std::error;
-use std::fmt;
-use std::marker::PhantomData;
+pub mod types;
 
-use crate::backend::shader::{Shader, Uniformable};
-use crate::context::GraphicsContext;
-use crate::vertex::Semantics;
+use crate::{
+  backend::shader::{Shader, ShaderData as ShaderDataBackend, Uniformable},
+  context::GraphicsContext,
+  vertex::Semantics,
+};
+use std::{error, fmt, marker::PhantomData};
 
 /// A shader stage type.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
@@ -337,6 +323,7 @@ impl From<ProgramWarning> for ProgramError {
 pub enum UniformWarning {
   /// Inactive uniform (not in use / no participation to the final output in shaders).
   Inactive(String),
+
   /// Type mismatch between the static requested type (i.e. the `T` in [`Uniform<T>`] for instance)
   /// and the type that got reflected from the backend in the shaders.
   ///
@@ -344,11 +331,27 @@ pub enum UniformWarning {
   ///
   /// [`Uniform<T>`]: crate::shader::Uniform
   TypeMismatch(String, UniformType),
+
   /// The requested type is unsupported by the backend.
   ///
   /// The [`String`] is the name of the uniform. The [`UniformType`] is the type that is not
   /// supported by the backend.
   UnsupportedType(String, UniformType),
+
+  /// Size mismatch between the static requested type (i.e. the `T` in [`Uniform<T>`] for instance)
+  /// and the size that got reflected from the backend in the shaders.
+  ///
+  /// [`Uniform<T>`]: crate::shader::Uniform
+  SizeMismatch {
+    /// Name of the uniform.
+    name: String,
+
+    /// Size of the uniform (static).
+    size: usize,
+
+    /// Found size of the uniform (in the shader).
+    found_size: usize,
+  },
 }
 
 impl UniformWarning {
@@ -375,6 +378,15 @@ impl UniformWarning {
   {
     UniformWarning::UnsupportedType(name.into(), ty)
   }
+
+  /// Create a size mismatch error.
+  pub fn size_mismatch(name: impl Into<String>, size: usize, found_size: usize) -> Self {
+    UniformWarning::SizeMismatch {
+      name: name.into(),
+      size,
+      found_size,
+    }
+  }
 }
 
 impl fmt::Display for UniformWarning {
@@ -389,6 +401,18 @@ impl fmt::Display for UniformWarning {
       UniformWarning::UnsupportedType(ref name, ref ty) => {
         write!(f, "unsupported type {} for uniform {}", ty, name)
       }
+
+      UniformWarning::SizeMismatch {
+        ref name,
+        size,
+        found_size,
+      } => {
+        write!(
+          f,
+          "size mismatch for uniform {}: {} (tdetected size={}",
+          name, size, found_size
+        )
+      }
     }
   }
 }
@@ -436,8 +460,9 @@ impl error::Error for VertexAttribWarning {}
 ///
 /// A uniform is a special variable that can be used to send data to a GPU. Several
 /// forms exist, but the idea is that `T` represents the data you want to send. Some exceptions
-/// exist that allow to pass _indirect_ data — such as [`BufferBinding`] to pass a buffer, or
-/// [`TextureBinding`] to pass a texture in order to fetch from it in a shader stage.
+/// exist that allow to pass shared data — such as [`ShaderDataBinding`] to pass a
+/// [`ShaderData`], or [`TextureBinding`] to pass a [`Texture`] in order to fetch from it in a
+/// shader stage.
 ///
 /// You will never be able to store them by your own. Instead, you must use a [`UniformInterface`],
 /// which provides a _contravariant_ interface for you. Creation is `unsafe` and should be
@@ -447,8 +472,9 @@ impl error::Error for VertexAttribWarning {}
 ///
 /// - `T` is the type of data you want to be able to set in a shader program.
 ///
-/// [`BufferBinding`]: crate::pipeline::BufferBinding
+/// [`Texture`]: crate::texture::Texture
 /// [`TextureBinding`]: crate::pipeline::TextureBinding
+/// [`ShaderDataBinding`]: crate::pipeline::ShaderDataBinding
 #[derive(Debug)]
 pub struct Uniform<T>
 where
@@ -588,9 +614,8 @@ pub enum UniformType {
   /// Floating-point cubemap sampler.
   Cubemap,
 
-  // buffer
-  /// Buffer binding; used for UBOs.
-  BufferBinding,
+  /// Shader data binding.
+  ShaderDataBinding,
 }
 
 impl fmt::Display for UniformType {
@@ -640,7 +665,7 @@ impl fmt::Display for UniformType {
       UniformType::ICubemap => f.write_str("isamplerCube"),
       UniformType::UICubemap => f.write_str("usamplerCube"),
       UniformType::Cubemap => f.write_str("samplerCube"),
-      UniformType::BufferBinding => f.write_str("buffer binding"),
+      UniformType::ShaderDataBinding => f.write_str("shader data binding"),
     }
   }
 }
@@ -712,22 +737,20 @@ where
   B: ?Sized + Shader,
 {
   /// Ask the creation of a [`Uniform`], identified by its `name`.
-  pub fn ask<T, N>(&mut self, name: N) -> Result<Uniform<T>, UniformWarning>
+  pub fn ask<T>(&mut self, name: &str) -> Result<Uniform<T>, UniformWarning>
   where
-    N: AsRef<str>,
-    T: Uniformable<B>,
+    B: for<'u> Uniformable<'u, T>,
   {
-    unsafe { B::ask_uniform(&mut self.repr, name.as_ref()) }
+    unsafe { B::ask_uniform(&mut self.repr, name) }
   }
 
   /// Ask the creation of a [`Uniform`], identified by its `name`.
   ///
   /// If the name is not found, an _unbound_ [`Uniform`] is returned (i.e. a [`Uniform`]) that does
   /// nothing.
-  pub fn ask_or_unbound<T, N>(&mut self, name: N) -> Uniform<T>
+  pub fn ask_or_unbound<T>(&mut self, name: &str) -> Uniform<T>
   where
-    N: AsRef<str>,
-    T: Uniformable<B>,
+    B: for<'u> Uniformable<'u, T>,
   {
     match self.ask(name) {
       Ok(uniform) => uniform,
@@ -762,7 +785,7 @@ where
 /// [luminance-derive]: https://crates.io/crates/luminance-derive
 pub trait UniformInterface<B, E = ()>: Sized
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   /// Create a [`UniformInterface`] by constructing [`Uniform`]s with a [`UniformBuilder`] and an
   /// optional environment object.
@@ -779,7 +802,7 @@ where
 
 impl<B, E> UniformInterface<B, E> for ()
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   fn uniform_interface<'a>(
     _: &mut UniformBuilder<'a, B>,
@@ -801,7 +824,7 @@ where
 /// - `Uni` is the [`UniformInterface`] type.
 pub struct BuiltProgram<B, Sem, Out, Uni>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   /// Built program.
   pub program: Program<B, Sem, Out, Uni>,
@@ -811,7 +834,7 @@ where
 
 impl<B, Sem, Out, Uni> BuiltProgram<B, Sem, Out, Uni>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   /// Get the program and ignore the warnings.
   pub fn ignore_warnings(self) -> Program<B, Sem, Out, Uni> {
@@ -829,7 +852,7 @@ where
 /// - `Uni` is the [`UniformInterface`] type.
 pub struct AdaptationFailure<B, Sem, Out, Uni>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   /// Program used before trying to adapt.
   pub program: Program<B, Sem, Out, Uni>,
@@ -839,7 +862,7 @@ where
 
 impl<B, Sem, Out, Uni> AdaptationFailure<B, Sem, Out, Uni>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   pub(crate) fn new(program: Program<B, Sem, Out, Uni>, error: ProgramError) -> Self {
     AdaptationFailure { program, error }
@@ -864,21 +887,25 @@ where
 /// `B` is the backend type.
 pub struct ProgramInterface<'a, B>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   pub(crate) program: &'a mut B::ProgramRepr,
 }
 
 impl<'a, B> ProgramInterface<'a, B>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   /// Set a value on a [`Uniform`].
-  pub fn set<T>(&mut self, uniform: &Uniform<T>, value: T)
+  ///
+  /// The value that is passed depends on the associated [`Uniformable::Target`] type. Most of the time, it will be the
+  /// same as `T`, but it might sometimes be something different if you are using existential types, such as with types
+  /// with lifetimes.
+  pub fn set<'u, T>(&'u mut self, uniform: &'u Uniform<T>, value: B::Target)
   where
-    T: Uniformable<B>,
+    B: Uniformable<'u, T>,
   {
-    unsafe { T::update(value, self.program, uniform) };
+    unsafe { B::update(self.program, uniform, value) };
   }
 
   /// Get back a [`UniformBuilder`] to dynamically access [`Uniform`] objects.
@@ -1119,7 +1146,7 @@ where
 /// - `Uni` is the [`UniformInterface`] type.
 pub struct Program<B, Sem, Out, Uni>
 where
-  B: ?Sized + Shader,
+  B: Shader,
 {
   pub(crate) repr: B::ProgramRepr,
   pub(crate) uni: Uni,
@@ -1129,7 +1156,7 @@ where
 
 impl<B, Sem, Out, Uni> Program<B, Sem, Out, Uni>
 where
-  B: ?Sized + Shader,
+  B: Shader,
   Sem: Semantics,
 {
   /// Create a new [`UniformInterface`] but keep the [`Program`] around without rebuilding it.
@@ -1211,3 +1238,90 @@ where
     self.adapt_env(env)
   }
 }
+
+/// Shader data.
+///
+/// # Parametricity
+///
+/// - `B` is the backend type.
+/// - `T` is the type of the carried items.
+pub struct ShaderData<B, T>
+where
+  B: ?Sized + ShaderDataBackend<T>,
+{
+  pub(crate) repr: B::ShaderDataRepr,
+}
+
+impl<B, T> ShaderData<B, T>
+where
+  B: ?Sized + ShaderDataBackend<T>,
+{
+  /// Create a [`ShaderData`] via an iterator of values.
+  pub fn new(
+    ctx: &mut impl GraphicsContext<Backend = B>,
+    values: impl IntoIterator<Item = T>,
+  ) -> Result<Self, ShaderDataError> {
+    let repr = unsafe { ctx.backend().new_shader_data(values.into_iter())? };
+    Ok(Self { repr })
+  }
+
+  /// Get the value at index `i`.
+  pub fn at(&self, i: usize) -> Result<T, ShaderDataError> {
+    unsafe { B::get_shader_data_at(&self.repr, i) }
+  }
+
+  /// Set the item at index `i` with the value `x`.
+  ///
+  /// Return the previous value.
+  pub fn set(&mut self, i: usize, x: T) -> Result<T, ShaderDataError> {
+    unsafe { B::set_shader_data_at(&mut self.repr, i, x) }
+  }
+
+  /// Replace all the values with the one provided by the iterator.
+  pub fn replace(&mut self, values: impl IntoIterator<Item = T>) -> Result<(), ShaderDataError> {
+    unsafe { B::set_shader_data_values(&mut self.repr, values.into_iter()) }
+  }
+}
+
+/// Possible errors that can occur with shader data.
+#[non_exhaustive]
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub enum ShaderDataError {
+  /// Cannot create the shader data on the backend side.
+  CannotCreate,
+
+  /// Index out of bounds.,
+  OutOfBounds {
+    /// Tried (incorrect) index.
+    index: usize,
+  },
+
+  /// Cannot set data.
+  CannotSetData {
+    /// Tried (incorrect) index.
+    index: usize,
+  },
+
+  /// Cannot replace data.
+  CannotReplaceData,
+}
+
+impl fmt::Display for ShaderDataError {
+  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
+    match self {
+      ShaderDataError::CannotCreate => f.write_str("cannot create shader data"),
+
+      ShaderDataError::OutOfBounds { index } => {
+        write!(f, "cannot get shader data item; out of bounds {}", index)
+      }
+
+      ShaderDataError::CannotSetData { index } => {
+        write!(f, "cannot get shader data item; out of bounds {}", index)
+      }
+
+      ShaderDataError::CannotReplaceData => f.write_str("cannot replace shader data"),
+    }
+  }
+}
+
+impl std::error::Error for ShaderDataError {}
